package com.slibra.web.controller.business;

import java.util.List;
import javax.servlet.http.HttpServletResponse;

import com.slibra.business.domain.TCloseTime;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import com.slibra.common.annotation.Log;
import com.slibra.common.core.controller.BaseController;
import com.slibra.common.core.domain.AjaxResult;
import com.slibra.common.enums.BusinessType;
import com.slibra.business.domain.TWarningRecord;
import com.slibra.business.service.ITWarningRecordService;
import com.slibra.common.utils.poi.ExcelUtil;
import com.slibra.common.core.page.TableDataInfo;

/**
 * 告警记录Controller
 * 
 * @author slibra
 * @date 2025-01-16
 */
@RestController
@RequestMapping("/business/warningRecord")
public class TWarningRecordController extends BaseController
{
    @Autowired
    private ITWarningRecordService tWarningRecordService;

    /**
     * 查询告警记录列表
     */
    @GetMapping("/list")
    public TableDataInfo list(TWarningRecord tWarningRecord)
    {
        startPage();
        List<TWarningRecord> list = tWarningRecordService.selectTWarningRecordList(tWarningRecord);
        return getDataTable(list);
    }

    /**
     * 导出告警记录列表
     */
    @PostMapping("/export")
    public void export(HttpServletResponse response, TWarningRecord tWarningRecord)
    {
        List<TWarningRecord> list = tWarningRecordService.selectTWarningRecordList(tWarningRecord);
        ExcelUtil<TWarningRecord> util = new ExcelUtil<TWarningRecord>(TWarningRecord.class);
        util.exportExcel(response, list, "告警记录数据");
    }

    /**
     * 获取告警记录详细信息
     */
    @GetMapping(value = "/{id}")
    public AjaxResult getInfo(@PathVariable("id") Long id)
    {
        return success(tWarningRecordService.selectTWarningRecordById(id));
    }

    /**
     * 新增告警记录
     */
    @PostMapping
    public AjaxResult add(@RequestBody TWarningRecord tWarningRecord)
    {
        tWarningRecord.setCreateBy(getUsername());
        return toAjax(tWarningRecordService.insertTWarningRecord(tWarningRecord));
    }

    /**
     * 修改告警记录
     */
    @PutMapping
    public AjaxResult edit(@RequestBody TWarningRecord tWarningRecord)
    {
        tWarningRecord.setUpdateBy(getUsername());
        return toAjax(tWarningRecordService.updateTWarningRecord(tWarningRecord));
    }

    /**
     * 删除告警记录
     */
	@DeleteMapping("/{ids}")
    public AjaxResult remove(@PathVariable Long[] ids)
    {
        return toAjax(tWarningRecordService.deleteTWarningRecordByIds(ids));
    }


    //------------下面是 用户关闭弹出记录时间接口  和   通过查询关闭弹出记录，获取最新的还未读的报警记录的条数

    /**
     * 用户关闭弹出记录时间接口
     */
    @PostMapping("/addCloseRecord")
    public AjaxResult addCloseRecord()
    {
        return success(tWarningRecordService.addCloseRecord());
    }


    /**
     * 获取上次关闭报警弹出以后，最新的还未读的报警记录的条数
     */
    @GetMapping(value = "/getNewestWarningCount")
    public AjaxResult getNewestWarningCount()
    {
        return success(tWarningRecordService.getNewestWarningCount());
    }
}
