package com.slibra.common.utils;

import java.lang.management.ManagementFactory;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.time.*;
import java.time.format.DateTimeFormatter;
import java.time.format.DateTimeParseException;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.List;

import org.apache.commons.lang3.time.DateFormatUtils;

import static com.slibra.common.constant.MyConstants.DEFAULT_DAYS_QUERY_COUNT;

/**
 * 时间工具类
 * 
 *
 */
public class DateUtils extends org.apache.commons.lang3.time.DateUtils
{
    public static String YYYY = "yyyy";

    public static String YYYY_MM = "yyyy-MM";

    public static String YYYY_MM_DD = "yyyy-MM-dd";

    public static String YYYY_MM_DD_HH = "yyyy-MM-dd HH";
    public static String YYYY_MM_DD_HH_MM = "yyyy-MM-dd HH:mm";

    public static String YYYYMMDDHHMMSS = "yyyyMMddHHmmss";

    public static String YYYY_MM_DD_HH_MM_SS = "yyyy-MM-dd HH:mm:ss";

    public static String YYYYMMDDHHMM_TS = "yyyy/MM/dd HH:mm";
    public static String YYYYMMDDHH_TS = "yyyy/MM/dd HH";
    public static String YYYYMMDD_TS = "yyyy/MM/dd";
    public static String YYYYMM_TS = "yyyy/MM";
    public static String YYYYMMDDHH_CHINESE = "yyyy年MM月dd日 HH";
    public static String YYYY_MM_DD_HH_MM_SSS = "yyyy-MM-dd HH:mm:ss.SSS";

    private static String[] parsePatterns = {
            "yyyy-MM-dd", "yyyy-MM-dd HH:mm:ss", "yyyy-MM-dd HH:mm", "yyyy-MM", 
            "yyyy/MM/dd", "yyyy/MM/dd HH:mm:ss", "yyyy/MM/dd HH:mm", "yyyy/MM",
            "yyyy.MM.dd", "yyyy.MM.dd HH:mm:ss", "yyyy.MM.dd HH:mm", "yyyy.MM"};

    /**
     * 获取当前Date型日期
     * 
     * @return Date() 当前日期
     */
    public static Date getNowDate()
    {
        return new Date();
    }

    /**
     * 获取当前日期, 默认格式为yyyy-MM-dd
     * 
     * @return String
     */
    public static String getDate()
    {
        return dateTimeNow(YYYY_MM_DD);
    }

    public static final String getTime()
    {
        return dateTimeNow(YYYY_MM_DD_HH_MM_SS);
    }

    public static final String dateTimeNow()
    {
        return dateTimeNow(YYYYMMDDHHMMSS);
    }

    public static final String dateTimeNow(final String format)
    {
        return parseDateToStr(format, new Date());
    }

    public static final String dateTime(final Date date)
    {
        return parseDateToStr(YYYY_MM_DD, date);
    }

    public static final String parseDateToStr(final String format, final Date date)
    {
        return new SimpleDateFormat(format).format(date);
    }

    public static final Date dateTime(final String format, final String ts)
    {
        try
        {
            return new SimpleDateFormat(format).parse(ts);
        }
        catch (ParseException e)
        {
            throw new RuntimeException(e);
        }
    }

    /**
     * 日期路径 即年/月/日 如2018/08/08
     */
    public static final String datePath()
    {
        Date now = new Date();
        return DateFormatUtils.format(now, "yyyy/MM/dd");
    }

    /**
     * 日期路径 即年/月/日 如20180808
     */
    public static final String dateTime()
    {
        Date now = new Date();
        return DateFormatUtils.format(now, "yyyyMMdd");
    }

    /**
     * 日期型字符串转化为日期 格式
     */
    public static Date parseDate(Object str)
    {
        if (str == null)
        {
            return null;
        }
        try
        {
            return parseDate(str.toString(), parsePatterns);
        }
        catch (ParseException e)
        {
            return null;
        }
    }

    /**
     * 获取服务器启动时间
     */
    public static Date getServerStartDate()
    {
        long time = ManagementFactory.getRuntimeMXBean().getStartTime();
        return new Date(time);
    }

    /**
     * 计算相差天数
     */
    public static int differentDaysByMillisecond(Date date1, Date date2)
    {
        return Math.abs((int) ((date2.getTime() - date1.getTime()) / (1000 * 3600 * 24)));
    }



    /**
     * 计算相差小时数
     */
    public static int differentHoursByMillisecond(Date date1, Date date2)
    {
        return Math.abs((int) ((date2.getTime() - date1.getTime()) / (1000 * 3600)));
    }

    /**
     * 计算时间差
     *
     * @param endDate 最后时间
     * @param startTime 开始时间
     * @return 时间差（天/小时/分钟）
     */
    public static String timeDistance(Date endDate, Date startTime)
    {
        long nd = 1000 * 24 * 60 * 60;
        long nh = 1000 * 60 * 60;
        long nm = 1000 * 60;
        // long ns = 1000;
        // 获得两个时间的毫秒时间差异
        long diff = endDate.getTime() - startTime.getTime();
        // 计算差多少天
        long day = diff / nd;
        // 计算差多少小时
        long hour = diff % nd / nh;
        // 计算差多少分钟
        long min = diff % nd % nh / nm;
        // 计算差多少秒//输出结果
        // long sec = diff % nd % nh % nm / ns;
        return day + "天" + hour + "小时" + min + "分钟";
    }

    /**
     * 增加 LocalDateTime ==> Date
     */
    public static Date toDate(LocalDateTime temporalAccessor)
    {
        ZonedDateTime zdt = temporalAccessor.atZone(ZoneId.systemDefault());
        return Date.from(zdt.toInstant());
    }

    /**
     * 增加 LocalDate ==> Date
     */
    public static Date toDate(LocalDate temporalAccessor)
    {
        LocalDateTime localDateTime = LocalDateTime.of(temporalAccessor, LocalTime.of(0, 0, 0));
        ZonedDateTime zdt = localDateTime.atZone(ZoneId.systemDefault());
        return Date.from(zdt.toInstant());
    }


    public static Date plusDate(int day, Date current)
    {

        // 创建一个Calendar实例，并将日期设置为今天
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(current);
        // 将日期设置为前一天
        calendar.add(Calendar.DATE, day);

        // 转换
        return calendar.getTime();
    }


    public static String plusDate(int day, Date current, String format)
    {

        // 创建一个Calendar实例，并将日期设置为今天
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(current);
        // 将日期设置为前一天
        calendar.add(Calendar.DATE, day);

        // 转换
        return parseDateToStr(format, calendar.getTime());
    }


    public static List<String> getAllDatesBetween(LocalDate startDate, LocalDate endDate) {
        List<String> dates = new ArrayList<>();
        LocalDate currentDate = startDate;

        while (!currentDate.isAfter(endDate)) {
            dates.add(currentDate.format(DateTimeFormatter.ofPattern(DateUtils.YYYYMMDD_TS)));
            currentDate = currentDate.plusDays(1);
        }

        return dates;
    }

    /**
     * 获取n个月之前的YYYY_MM格式的月份数据，输入参数也为YYYY_MM
     * @param inputDate
     * @param n
     * @return
     */
    public static String getMonthsBefore(String inputDate, int n) {
        DateTimeFormatter formatter = DateTimeFormatter.ofPattern(YYYY_MM_DD);
        DateTimeFormatter formatterOut = DateTimeFormatter.ofPattern(YYYY_MM);
        LocalDate date = LocalDate.parse(inputDate + "-01", formatter.withZone(java.time.ZoneId.systemDefault()));
        // 获取 n 个月之前的日期
        LocalDate targetDate = date.minusMonths(n);
        // 格式化目标日期为 "yyyy-MM" 格式
        return targetDate.format(formatterOut);
    }


    public static String getNYearSameDay(String inputDate, int n) {
        DateTimeFormatter formatter = DateTimeFormatter.ofPattern(YYYY_MM_DD);
        LocalDate date = null;
        try {
            date = LocalDate.parse(inputDate, formatter);
        } catch (DateTimeParseException e) {
            System.out.println("输入的日期格式不正确，请使用yyyy-MM-dd格式。");
            return null;
        }
        LocalDate lastYearDate = date.minusYears(n);
        return lastYearDate.format(formatter);
    }


    public static String getNYearSameMonth(String inputDate, int n) {
        DateTimeFormatter formatter = DateTimeFormatter.ofPattern(YYYY_MM);
        try {
            // 解析输入的字符串为YearMonth对象
            YearMonth currentYearMonth = YearMonth.parse(inputDate, formatter);
            // 获取去年的同一月份
            YearMonth previousYearMonth = currentYearMonth.minusYears(n);
            // 格式化为字符串
            return previousYearMonth.format(formatter);
        } catch (DateTimeParseException e) {
            System.err.println("输入的日期格式不正确，请使用yyyy-MM格式。");
        }
        return null;
    }


    /**
     * 获取两个日期之间的所有日期，格式为yyyy-MM-dd的集合
     *
     * @param startDateStr 开始日期字符串，格式为yyyy-MM-dd
     * @param endDateStr  结束日期字符串，格式为yyyy-MM-dd
     * @return 包含所有日期的集合
     * @throws ParseException 如果日期字符串格式不正确
     */
    public static List<String> getAllDatesInRangeStr(String startDateStr, String endDateStr) {
        SimpleDateFormat sdf = new SimpleDateFormat(YYYY_MM_DD);
        Date startDate = null;
        Date endDate = null;
        try {
            startDate = sdf.parse(startDateStr);
            endDate = sdf.parse(endDateStr);
        } catch (ParseException e) {
            throw new RuntimeException(e);
        }
        List<String> dateList = new ArrayList<>();
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(startDate);

        // 添加开始日期
        dateList.add(sdf.format(calendar.getTime()));

        // 循环遍历日期范围
        while (calendar.getTime().before(endDate)) {
            calendar.add(Calendar.DAY_OF_MONTH, 1);
            dateList.add(sdf.format(calendar.getTime()));
        }
        return dateList;
    }


    public static List<String> getAllDatesInRangeDate(Date startDate, Date endDate) {
        SimpleDateFormat sdf = new SimpleDateFormat(YYYY_MM_DD);
        List<String> dateList = new ArrayList<>();
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(startDate);

        // 添加开始日期
        dateList.add(sdf.format(calendar.getTime()));

        // 循环遍历日期范围
        while (calendar.getTime().before(endDate)) {
            calendar.add(Calendar.DAY_OF_MONTH, 1);
            dateList.add(sdf.format(calendar.getTime()));
        }
        return dateList;
    }



    public static List<String> getMonthsBetween(String startDateStr, String endDateStr) {
        DateTimeFormatter formatter = DateTimeFormatter.ofPattern(YYYY_MM);
        YearMonth startDate = YearMonth.parse(startDateStr, formatter);
        YearMonth endDate = YearMonth.parse(endDateStr, formatter);

        List<String> monthsBetween = new ArrayList<>();
        YearMonth currentMonth = startDate;

        while (!currentMonth.isAfter(endDate)) {
            monthsBetween.add(currentMonth.format(formatter));
            currentMonth = currentMonth.plusMonths(1);
        }

        return monthsBetween;
    }


    public static void main(String[] args) {
        Date nowDate = getNowDate();
        //近七日呼入电话量趋势
        String date = DateUtils.getDate();
        String sevenDaysAgo = DateUtils.plusDate(-DEFAULT_DAYS_QUERY_COUNT, nowDate, YYYY_MM_DD);
        List<String> allDatesInRangeStr = DateUtils.getAllDatesInRangeStr(sevenDaysAgo, date);
        for (String todayYearDate : allDatesInRangeStr) {
            System.out.println(todayYearDate);
        }
        System.out.println("11111111");
        //近七日呼入电话量趋势--去年数据
        String dateLastYear = DateUtils.getNYearSameDay(date, 1);
        String sevenDaysAgoLastYear = DateUtils.getNYearSameDay(sevenDaysAgo, 1);
        List<String> allDatesInRangeStrLastYear = DateUtils.getAllDatesInRangeStr(sevenDaysAgoLastYear, dateLastYear);
        for (String lastYearDate : allDatesInRangeStrLastYear) {
            System.out.println(lastYearDate);
        }
        System.out.println("22222222");
        //近7个月电话量同比趋势
        //获取当前的月份
        String nowMonth = DateUtils.parseDateToStr(DateUtils.YYYY_MM, nowDate);
        String sevenMonthAgo = DateUtils.getMonthsBefore(nowMonth, DEFAULT_DAYS_QUERY_COUNT);
        List<String> todayMonths = DateUtils.getMonthsBetween(sevenMonthAgo, nowMonth);
        for (String todayMonth : todayMonths) {
            System.out.println(todayMonth);
        }
        System.out.println("333333333");
        //近7个月电话量同比趋势--去年数据
        String nowMonthLastYear = DateUtils.getNYearSameMonth(nowMonth, 1);
        String sevenMonthAgoLastYear = DateUtils.getMonthsBefore(nowMonthLastYear, DEFAULT_DAYS_QUERY_COUNT);
        List<String> lastYearMonths = DateUtils.getMonthsBetween(sevenMonthAgoLastYear, nowMonthLastYear);
        for (String lastYearMonth : lastYearMonths) {
            System.out.println(lastYearMonth);
        }

    }
}
