package com.slibra.business.service.impl;

import java.io.File;
import java.math.BigDecimal;
import java.util.*;

import com.alibaba.fastjson2.JSON;
import com.slibra.business.domain.TBucketRecord;
import com.slibra.business.domain.TCutOffWater;
import com.slibra.business.domain.TUserInfo;
import com.slibra.business.mapper.TBucketRecordMapper;
import com.slibra.business.mapper.TCutOffWaterMapper;
import com.slibra.business.mapper.TUserInfoMapper;
import com.slibra.business.res.*;
import com.slibra.common.DecimalUtils;
import com.slibra.common.core.domain.entity.SysUser;
import com.slibra.common.exception.ServiceException;
import com.slibra.common.utils.DateUtils;
import com.slibra.common.utils.StringUtils;
import com.slibra.system.mapper.SysUserMapper;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import com.slibra.business.mapper.TCallRecordMapper;
import com.slibra.business.domain.TCallRecord;
import com.slibra.business.service.ITCallRecordService;
import org.springframework.util.CollectionUtils;

import static com.slibra.common.constant.MyConstants.*;
import static com.slibra.common.utils.DateUtils.YYYY_MM_DD;

/**
 * 通话记录Service业务层处理
 * 
 * @author slibra
 * @date 2024-11-11
 */
@Service
@Slf4j
public class TCallRecordServiceImpl implements ITCallRecordService 
{
    @Autowired
    private TCallRecordMapper tCallRecordMapper;

    @Autowired
    private TUserInfoMapper tUserInfoMapper;

    @Autowired
    private TCutOffWaterMapper cutOffWaterMapper;

    @Value("${ALIYUN_AK_ID}")
    private String accessKeyId;

    @Value("${ALIYUN_AK_SECRET}")
    private String accessKeySecret;

    @Value("${NLS_APP_KEY}")
    private String appKey;

    @Value("${WS_URL}")
    private String wsUrl;

    /*@Value("${DOMAIN}")
    private String domain;

    @Value("${URL_DISK_PREFIX}")
    private String urlDiskPrefix;*/

    @Value("${FILE_URL_PREFIX}")
    private String fileUrlPrefix;

    @Autowired
    @Qualifier("qiNiuYunServiceNew")
    QiNiuYunServiceImpl qiNiuYunService;

    @Autowired
    private SysUserMapper sysUserMapper;

    @Autowired
    private TBucketRecordMapper bucketRecordMapper;

    /**
     * 查询通话记录
     * 
     * @param id 通话记录主键
     * @return 通话记录
     */
    @Override
    public TCallRecord selectTCallRecordById(Long id)
    {
        TCallRecord tCallRecord = tCallRecordMapper.selectTCallRecordById(id);
        if(!Objects.isNull(tCallRecord))
            this.handleUserName(tCallRecord);
        return tCallRecord;
    }

    /**
     * 查询通话记录列表
     * 
     * @param tCallRecord 通话记录
     * @return 通话记录
     */
    @Override
    public List<TCallRecord> selectTCallRecordList(TCallRecord tCallRecord)
    {
        List<TCallRecord> tCallRecords = tCallRecordMapper.selectTCallRecordList(tCallRecord);
        if(!CollectionUtils.isEmpty(tCallRecords)){
            for (TCallRecord callRecord : tCallRecords) {
                this.handleUserName(callRecord);
            }
        }
        return tCallRecords;
    }

    private void handleUserName(TCallRecord callRecord) {
        Long serviceCategory = callRecord.getServiceCategory();
        if(!Objects.isNull(serviceCategory) && 2L == serviceCategory){
            String transferUserName = callRecord.getTransferUserName();//有可能不存在  如果存在，再替换
            if(!StringUtils.isEmpty(transferUserName)){
                callRecord.setUserName(transferUserName);
            }
        }
    }

    /**
     * 新增通话记录
     * 
     * @param tCallRecord 通话记录
     * @return 结果
     */
    @Override
    public int insertTCallRecord(TCallRecord tCallRecord)
    {
        tCallRecord.setCreateTime(DateUtils.getNowDate());
        return tCallRecordMapper.insertTCallRecord(tCallRecord);
    }

    /**
     * 修改通话记录
     * 
     * @param tCallRecord 通话记录
     * @return 结果
     */
    @Override
    public int updateTCallRecord(TCallRecord tCallRecord)
    {
        tCallRecord.setUpdateTime(DateUtils.getNowDate());
        //2024年11月18日10:23:18 由于前端不方便穿ID和name，这里由后台来操作
        //2025年01月06日18:24:10 这里不要覆盖以前记录的数据
        /*try {
            tCallRecord.setUserId(SecurityUtils.getUserId());
            tCallRecord.setUserName(SecurityUtils.getUsername());
        } catch (Exception e) {
            log.error("××××××跟新通话记录的时候，获取用户信息失败！");
        }*/
        //额外再查询曹永创的botrecords表，通过session获取intent，更新【业务类型】字段
        String sessionId = tCallRecord.getSessionId();
        if(StringUtils.isBlank(sessionId))
            log.error("更新通话记录的时候，由于获取不到sessionId，导致无法更新曹永创的字段bussinessType【业务类型】");
        else
            tCallRecord.setBussinessType(this.tCallRecordMapper.getBusinessTypeBySessionId(sessionId));
        return tCallRecordMapper.updateTCallRecord(tCallRecord);
    }

    /**
     * 批量删除通话记录
     * 
     * @param ids 需要删除的通话记录主键
     * @return 结果
     */
    @Override
    public int deleteTCallRecordByIds(Long[] ids)
    {
        return tCallRecordMapper.deleteTCallRecordByIds(ids);
    }

    /**
     * 删除通话记录信息
     * 
     * @param id 通话记录主键
     * @return 结果
     */
    @Override
    public int deleteTCallRecordById(Long id)
    {
        return tCallRecordMapper.deleteTCallRecordById(id);
    }

    @Override
    public TCallRecord getCallBySessionId(String sessionId) {
        if(StringUtils.isBlank(sessionId))
            throw new ServiceException("请输入sessionId");
        List<TCallRecord> tCallRecords = this.tCallRecordMapper.selectTCallRecordList(TCallRecord.builder().sessionId(sessionId).build());
        if(!CollectionUtils.isEmpty(tCallRecords)){
            TCallRecord tCallRecord = tCallRecords.get(0);
            tCallRecord.setCurrentCallRecords(this.tCallRecordMapper.getContentsBySessionId(sessionId));//[当前通话记录]
            this.addExtra(tCallRecord);
            return tCallRecord;
        }
        return null;
    }

    private void addExtra(TCallRecord tCallRecord) {
        String phone = tCallRecord.getPhone();
        //获取历史通话记录（不包含本次）
        List<TCallRecord> histories = this.tCallRecordMapper.selectHistoriesByPhone(phone, tCallRecord.getId());
        /*tCallRecord.setCounts(this.tCallRecordMapper.getCallCountsByPhone(phone));*/
        //户号水费小区名称
        List<TUserInfo> tUserInfos = this.tUserInfoMapper.selectTUserInfoList(TUserInfo.builder().phone(phone).build());
        tCallRecord.setUserInfos(tUserInfos);
        tCallRecord.setCallRecords(histories);
        tCallRecord.setCounts(histories.size());
        //正在进行的停水信息
        if(!CollectionUtils.isEmpty(tUserInfos)){
            List<TCutOffWater> cutOffWaters = new ArrayList<>();
            for (TUserInfo tUserInfo : tUserInfos) {
                cutOffWaters.addAll(this.cutOffWaterMapper.getCutOffWaterByNeighbourhoodName(tUserInfo.getNeighbourhoodName()));
            }
            tCallRecord.setCutOffWaters(cutOffWaters);
        }
    }

    @Override
    public List<UserTypeResp> getAgentList() {
        /*List<UserTypeResp> agentList = this.tCallRecordMapper.getAgentList();
        if(!CollectionUtils.isEmpty(agentList)){
            for (UserTypeResp userTypeResp : agentList) {
                Long serviceCategory = userTypeResp.getServiceCategory();
                if(!Objects.isNull(serviceCategory) && 0L == serviceCategory){//人工 需要特殊处理
                    Long id = userTypeResp.getId();
                    if(!Objects.isNull(id)){
                        SysUser sysUser = this.sysUserMapper.selectUserById(id);
                        if(!Objects.isNull(sysUser))
                            userTypeResp.setName(sysUser.getNickName());
                    }
                }
            }
        }
        return agentList;*/
        //2025年01月08日14:00:15  因为人工坐席已经全部记录下来了，无需从通话记录再获取
        List<UserTypeResp> agentList = this.tCallRecordMapper.getAgentList();
        //人工姓名再外处理下，防止有其他修改昵称的地方而没有处理
        if(!CollectionUtils.isEmpty(agentList)){
            for (UserTypeResp userTypeResp : agentList) {
                Integer type = userTypeResp.getType();
                if(!Objects.isNull(type) && 0 == type){//人工 需要特殊处理
                    Long id = userTypeResp.getId();
                    if(!Objects.isNull(id)){
                        SysUser sysUser = this.sysUserMapper.selectUserById(id);
                        if(!Objects.isNull(sysUser))
                            userTypeResp.setName(sysUser.getNickName());
                    }
                }
            }
        }
        return agentList;
    }

    /**
     * 2024年12月09日13:41:48 逻辑调整：获取通话记录，找到服务器上的录音文件，再上传到七牛云上(目前是存储了一天的时间)，然后再调用离线版本的语音解析，再返回给前端
     * @param id
     * @return
     */
    @Override
    public String updateFile2TextById(Long id) {
        //查询数据 校验数据
        TCallRecord tCallRecord = this.tCallRecordMapper.selectTCallRecordById(id);
        String url = checkParamsAndGetUrl(tCallRecord);
        log.info("服务区上存储录音文件是{}", url);
        url = fileUrlPrefix + url;
        log.info("拼接完前缀以后，文件的URL是{}", url);
//        url = "/Users/wangmiaomiao/Downloads/C1865941831922094080_0.wav";//本地测试
        //上传到七牛云
        File file = new File(url);
        url = this.qiNiuYunService.uploadQNImg(file);
        log.info("上传到七牛云以后的文件地址为{}", url);
        /*final String accessKeyId = "LTAI5tQ2HmiHCygZkt5BYrYR";
        final String accessKeySecret = "KhmxTd14SUcXafpFk5yofA43FoeM99";
        final String appKey = "OKt6jogp6fRjHQVp";*/
        File2Text file2Text = new File2Text(accessKeyId, accessKeySecret);
        // 第一步：提交录音文件识别请求，获取任务ID用于后续的识别结果轮询。
        String taskId = file2Text.submitFileTransRequest(appKey, url);
        if (taskId != null) {
            log.info("录音文件识别请求成功，task_id: " + taskId);
        }
        else {
            log.error("录音文件识别请求失败！");
//            return "";
            throw new ServiceException("录音文件识别请求失败！");
        }
        // 第二步：根据任务ID轮询识别结果。
        String result = file2Text.getFileTransResult(taskId);
        if (result != null) {
            log.info("录音文件识别结果查询成功：" + result);
//            Ele ele = JSON.parseObject(JSON.parseArray(JSON.parseObject(result).get("Sentences").toString()).get(0).toString(), Ele.class);
//            log.info(JSON.toJSONString(ele));
//            String text = ele.getText();
            //2024年11月19日11:09:33  不用再解析Sentences下面的某个值，把返回的Sentences全部返回给前端
            String sentences = null;
            try {
                sentences = JSON.parseObject(result).get("Sentences").toString();
            } catch (Exception e) {
                log.error("解析返回的语音结果异常，返回结果为{}\n异常信息为{}", result, e.getMessage());
                throw new ServiceException("解析返回的语音结果异常");
            }
            //同时更新到数据库中去
            tCallRecord.setHasParsed(LONG_1);
            tCallRecord.setParsedVoiceContent(sentences);
            this.tCallRecordMapper.updateTCallRecord(tCallRecord);
            return sentences;
        }
        else {
//            log.error("录音文件识别结果查询失败！");
            throw new ServiceException("录音文件识别结果查询失败！");
        }
    }


    @Override
    public String updateFile2TextByIdOneLine(Long id) {
        //查询数据 校验数据
        TCallRecord tCallRecord = this.tCallRecordMapper.selectTCallRecordById(id);
        String url = checkParamsAndGetUrl(tCallRecord);

        SpeechTranscriberDemo demo = new SpeechTranscriberDemo(appKey, accessKeyId, accessKeySecret, wsUrl);
//        demo.process(url.replace(domain, urlDiskPrefix));//假设是带http的，替换URL  示例：将https://pbx.fuxicarbon.com/voice/005cad2d-3626-477c-8b95-07b68f339630.wav 变化成 /root/aibot/dm/voice/005cad2d-3626-477c-8b95-07b68f339630.wav
        demo.shutdown();
        //同时更新到数据库中去
        tCallRecord.setHasParsed(LONG_1);
        String result = JSON.toJSONString(SpeechTranscriberDemo.result);
        tCallRecord.setParsedVoiceContent(result);
        this.tCallRecordMapper.updateTCallRecord(tCallRecord);
        return result;
    }

    @Override
    public CallRecordCount callRecordCountInfo() {
        CallRecordCount callRecordCount = CallRecordCount.builder().build();
        //处理通话统计信息
        int total = this.tCallRecordMapper.getTotalByType(null);
        callRecordCount.setTotal(total);
        int humanTotal = this.tCallRecordMapper.getTotalByType(2);
        callRecordCount.setHumanTotal(humanTotal);
        /*int whiteListTotal = this.tCallRecordMapper.getTotalByType(0);
        callRecordCount.setWhiteListTotal(whiteListTotal);*/
        int whiteListTotalHuman = this.tCallRecordMapper.getTotalByType(0);
        callRecordCount.setWhiteListTotalHuman(whiteListTotalHuman);
        int whiteListTotalRobot = this.tCallRecordMapper.getTotalByType(1);
        callRecordCount.setWhiteListTotalRobot(whiteListTotalRobot);
        int aiTotal = this.tCallRecordMapper.getTotalByIsAi(1);
        callRecordCount.setAiTotal(aiTotal);
        int traditionTotal = this.tCallRecordMapper.getTotalByIsAi(0);
        callRecordCount.setTraditionTotal(traditionTotal);
        int traditionOutTotal = this.tCallRecordMapper.getTotalByIsAi(2);
        callRecordCount.setTraditionOutTotal(traditionOutTotal);
        if(total > 0){
            BigDecimal humanPercent = DecimalUtils.getNLengthHalfUp(new BigDecimal(humanTotal / (double) total).multiply(BigDecimal_100), INT_0);
            callRecordCount.setHumanPercent(humanPercent);
            BigDecimal aiPercent = DecimalUtils.getNLengthHalfUp(new BigDecimal(aiTotal / (double) total).multiply(BigDecimal_100), INT_0);
            callRecordCount.setAiPercent(aiPercent);
            //2024年12月19日15:11:01  这里的百分比逻辑改成：traditionTotal + traditionOutTotal
            BigDecimal traditionPercent = DecimalUtils.getNLengthHalfUp(BigDecimal.valueOf((traditionTotal + traditionOutTotal) / (double) total).multiply(BigDecimal_100), INT_0);
            callRecordCount.setTraditionPercent(traditionPercent);
            //2024年12月19日18:55:23 需要四个值加在一起是100%  所以这里要用100减去上面三个之和
//            callRecordCount.setWhiteListPercent(whiteListTotal / (double)total);
            callRecordCount.setWhiteListPercent(DecimalUtils.getNLengthHalfUp(BigDecimal_100.subtract(humanPercent).subtract(aiPercent).subtract(traditionPercent), INT_0));
        }
        callRecordCount.setSpecialCount(this.tCallRecordMapper.getSpecialCount());
        int transferCount = this.tCallRecordMapper.getTransferCount(null, null);
        callRecordCount.setTransferCount(transferCount);
        //处理人工客服状态统计信息
        callRecordCount.setTotalPerson(this.tCallRecordMapper.getPersonCountByState(null));
        callRecordCount.setIdlePerson(this.tCallRecordMapper.getPersonCountByState(2));
        callRecordCount.setBusyPerson(this.tCallRecordMapper.getPersonCountByState(1));
        callRecordCount.setOnlinePerson(this.tCallRecordMapper.getPersonCountByState(3));
        callRecordCount.setDialPerson(this.tCallRecordMapper.getPersonCountByState(5));

        //通话次数和通话时长
        callRecordCount.setPersonCount(this.tCallRecordMapper.getPersonCountRecordIsRobot(0));
        callRecordCount.setPersonCountOn(this.tCallRecordMapper.getPersonCountByType(1));//接通
        callRecordCount.setPersonCountOff(this.tCallRecordMapper.getPersonCountByType(0));//未接通
        callRecordCount.setPersonTotal(this.tCallRecordMapper.getPersonTotalRecordIsRobot(0, null));

        callRecordCount.setRobotCount(this.tCallRecordMapper.getPersonCountRecordIsRobot(1));
        callRecordCount.setRobotTotal(this.tCallRecordMapper.getPersonTotalRecordIsRobot(1, null));
        //转人工次数
        callRecordCount.setRobotTransfer(transferCount);

        return callRecordCount;
    }

    /**
     * 2024年12月31日14:07:53  新增累计统计
     * @return
     */
    @Override
    public List<CallUserCountInfo> userCallCount() {
        //先获取所有的人工信息 再查状态 再统计数据
        //2025年01月07日09:50:29 逻辑调整：因为人工会改名字，机器人客服不会，所以人工只获取ID，再额外查询一次最新的名字，机器人则不需要
//        List<CallUserCountInfo> result = this.tCallRecordMapper.selectAllUsersByType(0);
        List<CallUserCountInfo> result = this.tCallRecordMapper.selectAllPersonUserIds();
        String date = DateUtils.getDate();
        if(!CollectionUtils.isEmpty(result)){
            for (CallUserCountInfo callUserCountInfo : result) {
                if(Objects.isNull(callUserCountInfo)){
                    log.error("统计人工客服数据的时候，存在姓名和ID全部为空的异常数据，跳过此次统计");
                    continue;
                }
                try {
                    SysUser sysUser = this.sysUserMapper.selectUserById(Long.valueOf(callUserCountInfo.getUserId()));
                    if(!Objects.isNull(sysUser))
                        callUserCountInfo.setUserName(sysUser.getNickName());
                } catch (NumberFormatException e) {
                    log.error("统计人工客服信息时，获取人工信息失败，异常信息为{}", e.getMessage());
                }
                String userId = callUserCountInfo.getUserId();
                if(StringUtils.isNotBlank(userId)){
                    callUserCountInfo.setStatus(this.parseStatus(this.tCallRecordMapper.getStatusByUserId(userId)));
                    callUserCountInfo.setTotalTimes(this.tCallRecordMapper.getPersonTotalRecordIsRobot(0, userId));
                    callUserCountInfo.setInTodayCount(this.tCallRecordMapper.getCountByTypeAndDate(0, date, userId));
                    callUserCountInfo.setInAllCount(this.tCallRecordMapper.getCountByTypeAndDate(0, null, userId));
                    callUserCountInfo.setOutTodayCount(this.tCallRecordMapper.getCountByTypeAndDate(1, date, userId));
                    callUserCountInfo.setOutAllCount(this.tCallRecordMapper.getCountByTypeAndDate(1, null, userId));
                }
            }
        }
        /*CallUserCountInfo callUserCountInfo = new CallUserCountInfo();
        callUserCountInfo.setUserName("累计");
        callUserCountInfo.setTotalTimes(result.stream().map(CallUserCountInfo::getTotalTimes).mapToLong(Long::longValue).sum());
        callUserCountInfo.setInTodayCount(result.stream().map(CallUserCountInfo::getInTodayCount).mapToInt(Integer::intValue).sum());
        callUserCountInfo.setInAllCount(result.stream().map(CallUserCountInfo::getInAllCount).mapToInt(Integer::intValue).sum());
        callUserCountInfo.setOutTodayCount(result.stream().map(CallUserCountInfo::getOutTodayCount).mapToInt(Integer::intValue).sum());
        callUserCountInfo.setOutAllCount(result.stream().map(CallUserCountInfo::getOutAllCount).mapToInt(Integer::intValue).sum());
        result.add(callUserCountInfo);*/
        return result;
    }

    private String parseStatus(Integer status) {
        if(Objects.isNull(status))
            return "";
        switch (status) {
            case 0: return "签出";
            case 1: return "置忙";
            case 2: return "置闲";
            case 3: return "通话中";
            case 4: return "后处理";
            case 5: return "拨号中";
            default:return "";

        }
    }


    /**
     * 2024年12月31日14:07:53  新增累计统计
     * @return
     */
    @Override
    public List<CallUserCountInfo> robotCallCount() {
        //先获取所有的机器人信息 再查状态 再统计数据
        List<CallUserCountInfo> result = this.tCallRecordMapper.selectAllUsersByType(1);
        String date = DateUtils.getDate();
        if(!CollectionUtils.isEmpty(result)){
            for (CallUserCountInfo callUserCountInfo : result) {
                if(Objects.isNull(callUserCountInfo)){
                    log.error("统计机器人客服数据的时候，存在姓名和ID全部为空的异常数据，跳过此次统计");
                    continue;
                }
                String userId = callUserCountInfo.getUserId();
                if(StringUtils.isNotBlank(userId)){
//                    callUserCountInfo.setStatus(this.tCallRecordMapper.getStatusByUserId(userId));
//                    callUserCountInfo.setTotalTimes(this.tCallRecordMapper.getPersonTotalRecordIsRobot(1, userId));
                    callUserCountInfo.setInTodayCount(this.tCallRecordMapper.getCountByTypeAndDate(0, date, userId));
                    callUserCountInfo.setInAllCount(this.tCallRecordMapper.getCountByTypeAndDate(0, null, userId));
//                    callUserCountInfo.setOutTodayCount(this.tCallRecordMapper.getCountByTypeAndDate(1, date));
//                    callUserCountInfo.setOutAllCount(this.tCallRecordMapper.getCountByTypeAndDate(1, null));
                    callUserCountInfo.setTodayTransferCount(this.tCallRecordMapper.getTransferCount(userId, date));
                    callUserCountInfo.setTotalTransferCount(this.tCallRecordMapper.getTransferCount(userId, null));
                }
            }
        }
        /*CallUserCountInfo callUserCountInfo = new CallUserCountInfo();
        callUserCountInfo.setUserName("累计");
//        callUserCountInfo.setTotalTimes(result.stream().map(CallUserCountInfo::getTotalTimes).mapToLong(Long::longValue).sum());
        callUserCountInfo.setInTodayCount(result.stream().map(CallUserCountInfo::getInTodayCount).mapToInt(Integer::intValue).sum());
        callUserCountInfo.setInAllCount(result.stream().map(CallUserCountInfo::getInAllCount).mapToInt(Integer::intValue).sum());
//        callUserCountInfo.setOutTodayCount(result.stream().map(CallUserCountInfo::getOutTodayCount).mapToInt(Integer::intValue).sum());
//        callUserCountInfo.setOutAllCount(result.stream().map(CallUserCountInfo::getOutAllCount).mapToInt(Integer::intValue).sum());
        callUserCountInfo.setTodayTransferCount(result.stream().map(CallUserCountInfo::getTodayTransferCount).mapToInt(Integer::intValue).sum());
        callUserCountInfo.setTotalTransferCount(result.stream().map(CallUserCountInfo::getTotalTransferCount).mapToInt(Integer::intValue).sum());
        result.add(callUserCountInfo);*/
        return result;
    }


    @Override
    public List<CallRecordListResp> callRecordCountPageList(TCallRecord callRecordReq) {
        List<CallRecordListResp> result = new ArrayList<>();
        String timeBeginReq = callRecordReq.getTimeBeginReq();
        String timeEndReq = callRecordReq.getTimeEndReq();
        if(StringUtils.isBlank(timeBeginReq) || StringUtils.isBlank(timeEndReq))
          throw new ServiceException("起止日期必须填写");
        //获取2个时间段内的所有日期，经确到天，即【yyyy-MM-dd】格式的
        List<String> allDatesInRangeDate = DateUtils.getAllDatesInRangeStr(timeBeginReq, timeEndReq);
        if(!CollectionUtils.isEmpty(allDatesInRangeDate)){
            for (String date : allDatesInRangeDate) {
                CallRecordListResp callRecordListResp = new CallRecordListResp();
                callRecordListResp.setDate(date);
                //根据日期获取统计数据
                //请求参数type说明： 0：接通总量  1：未接通总量  2：机器人接听  3：机器人转人工  4：人工接听  5：机器人处理量  6：人工处理量
                //呼入总量
                int inTotal = this.tCallRecordMapper.getTotalInByType(null, date);
                callRecordListResp.setInTotal(inTotal);
                callRecordListResp.setSuccessTotal(this.tCallRecordMapper.getTotalInByType(0, date));
                callRecordListResp.setFailTotal(this.tCallRecordMapper.getTotalInByType(1, date));
                callRecordListResp.setRobotHearTotal(this.tCallRecordMapper.getTotalInByType(2, date));
                callRecordListResp.setTransferTotal(this.tCallRecordMapper.getTotalInByType(3, date));
                callRecordListResp.setHumanTotal(this.tCallRecordMapper.getTotalInByType(4, date));
                //机器人处理量
                int robotHandleTotal = this.tCallRecordMapper.getTotalInByType(5, date);
                callRecordListResp.setRobotHandleTotal(robotHandleTotal);
                //人工处理量
                int humanHandleTotal = this.tCallRecordMapper.getTotalInByType(6, date);
                callRecordListResp.setHumanHandleTotal(humanHandleTotal);
                //处理 占比
                if(inTotal > 0){
                    BigDecimal robotRate = DecimalUtils.getNLengthHalfUp(new BigDecimal(robotHandleTotal/(double)inTotal).multiply(BigDecimal_100), INT_2);
                    BigDecimal humanRate = DecimalUtils.getNLengthHalfUp(new BigDecimal(humanHandleTotal/(double)inTotal).multiply(BigDecimal_100), INT_2);
                    callRecordListResp.setRobotRate(robotRate);
                    callRecordListResp.setHumanRate(humanRate);
                    callRecordListResp.setFailRate(DecimalUtils.getNLengthHalfUp(BigDecimal_100.subtract(robotRate).subtract(humanRate), INT_2));
                }
                result.add(callRecordListResp);
            }
        }
        return result;
    }

    @Override
    public HomeCountResp homeScreenCount() {
        HomeCountResp build = HomeCountResp.builder().build();
        //获取7天前的日期
        String lastDate = DateUtils.plusDate(-DEFAULT_DAYS_QUERY_COUNT, DateUtils.getNowDate(), YYYY_MM_DD);
        //查询说明： type未空是查询所有呼入的 0：成功接通的
        int inTotal = this.tCallRecordMapper.getRecentCallRecordAfterSomeDay(null, lastDate);
        build.setInTotal(inTotal);
        int inTotalSuccess = this.tCallRecordMapper.getRecentCallRecordAfterSomeDay(0, lastDate);
        if(inTotal > 0)
            build.setSuccessRate(DecimalUtils.getNLengthHalfUp(new BigDecimal(inTotalSuccess/(double)inTotal).multiply(BigDecimal_100), INT_2));
        long inTimes = this.tCallRecordMapper.getRecentCallTimes(lastDate);
        build.setInTimes(inTimes);
        if(inTotalSuccess > 0)
            build.setInTimesAvg(inTimes/inTotalSuccess);
        build.setTotalTimes(this.tCallRecordMapper.getRecentCallTimes(null));
        build.setTotalCounts(this.tCallRecordMapper.getRecentCallRecordAfterSomeDay(null, null));
        //近七日呼入电话量趋势
        List<DayAndCountRes> recent7DayAndCounts = new ArrayList<>();
        List<DayAndCountRes> recent7DayAndCountsLastYear = new ArrayList<>();
        Date nowDate = DateUtils.getNowDate();
        String nowDateStr = DateUtils.getDate();
        String sevenDaysAgo = DateUtils.plusDate(-DEFAULT_DAYS_QUERY_COUNT, nowDate, YYYY_MM_DD);
        List<String> allDatesInRangeStr = DateUtils.getAllDatesInRangeStr(sevenDaysAgo, nowDateStr);
        for (String todayYearDate : allDatesInRangeStr) {
            DayAndCountRes dayAndCountRes = new DayAndCountRes();
            dayAndCountRes.setDate(todayYearDate);
            dayAndCountRes.setCount(this.tCallRecordMapper.getCallInEqualsDay(todayYearDate, null, null));
            recent7DayAndCounts.add(dayAndCountRes);
        }
        build.setRecent7DayAndCounts(recent7DayAndCounts);
        //近七日呼入电话量趋势--去年数据
        String dateLastYear = DateUtils.getNYearSameDay(nowDateStr, 1);
        String sevenDaysAgoLastYear = DateUtils.getNYearSameDay(sevenDaysAgo, 1);
        List<String> allDatesInRangeStrLastYear = DateUtils.getAllDatesInRangeStr(sevenDaysAgoLastYear, dateLastYear);
        for (String lastYearDate : allDatesInRangeStrLastYear) {
            DayAndCountRes dayAndCountRes = new DayAndCountRes();
            dayAndCountRes.setDate(lastYearDate);
            dayAndCountRes.setCount(this.tCallRecordMapper.getCallInEqualsDay(lastYearDate, null, null));
            recent7DayAndCountsLastYear.add(dayAndCountRes);
        }
        build.setRecent7DayAndCountsLastYear(recent7DayAndCountsLastYear);
        //近7个月电话量同比趋势
        List<DayAndCountRes> recent7MonthAndCounts = new ArrayList<>();
        //获取当前的月份
        String nowMonth = DateUtils.parseDateToStr(DateUtils.YYYY_MM, nowDate);
        String sevenMonthAgo = DateUtils.getMonthsBefore(nowMonth, DEFAULT_DAYS_QUERY_COUNT);
        List<String> todayMonths = DateUtils.getMonthsBetween(sevenMonthAgo, nowMonth);
        for (String todayMonth : todayMonths) {
            DayAndCountRes dayAndCountRes = new DayAndCountRes();
            dayAndCountRes.setDate(todayMonth);
            dayAndCountRes.setCount(this.tCallRecordMapper.getCallInEqualsMonth(todayMonth));
            recent7MonthAndCounts.add(dayAndCountRes);
        }
        build.setRecent7MonthAndCounts(recent7MonthAndCounts);
        //近7个月电话量同比趋势--去年数据
        List<DayAndCountRes> recent7MonthAndCountsLastYear = new ArrayList<>();
        String nowMonthLastYear = DateUtils.getNYearSameMonth(nowMonth, 1);
        String sevenMonthAgoLastYear = DateUtils.getMonthsBefore(nowMonthLastYear, DEFAULT_DAYS_QUERY_COUNT);
        List<String> lastYearMonths = DateUtils.getMonthsBetween(sevenMonthAgoLastYear, nowMonthLastYear);
        for (String lastYearMonth : lastYearMonths) {
            DayAndCountRes dayAndCountRes = new DayAndCountRes();
            dayAndCountRes.setDate(lastYearMonth);
            dayAndCountRes.setCount(this.tCallRecordMapper.getCallInEqualsMonth(lastYearMonth));
            recent7MonthAndCountsLastYear.add(dayAndCountRes);
        }
        build.setRecent7MonthAndCountsLastYear(recent7MonthAndCountsLastYear);
        //本月业务类型 top7 处理
        List<Top7Res> top7ResList = this.tCallRecordMapper.getTop7ResListByMonthAndSize(nowMonth, DEFAULT_DAYS_QUERY_COUNT+1);
        //处理展示数据
        if(!CollectionUtils.isEmpty(top7ResList)) {
            //获取本月的所有的数据
            int nowMonthSize = this.tCallRecordMapper.getCallInEqualsMonth(nowMonth);
            for (int i = 0; i < top7ResList.size(); i++) {
                Top7Res top7Res = top7ResList.get(i);
                top7Res.setIndex(i+1);
                if(nowMonthSize > 0)
                    top7Res.setPercent(DecimalUtils.getNLengthHalfUp(BigDecimal.valueOf(top7Res.getTotal() / (double) nowMonthSize).multiply(BigDecimal_100), INT_2));
            }
        }
        build.setBusinessTop7(top7ResList);
        //今日机器人坐席情况
        //type 0人工  1机器人  2转人工     通过service_category区分：0：0人工坐席    1：1机器人坐席 + 2机器人转人工  2：2机器人转人工
        //status 0未接听 1已接通  不传，默认所有
        build.setRobotInTotal(this.tCallRecordMapper.getCallInEqualsDay(nowDateStr, 1, null));
        long robotInTimes = this.tCallRecordMapper.getCallTimesEqualsDay(nowDateStr, 1);//0人工  1机器人  2转人工     通过service_category区分：0：0人工坐席    1：1机器人坐席 + 2机器人转人工  2：2机器人转人工
        build.setRobotInTimes(robotInTimes);
        //获取今日已接听的机器人坐席总呼入量
        int callInTodaySuccess = this.tCallRecordMapper.getCallInEqualsDay(nowDateStr, 1, 1);
        if(callInTodaySuccess > 0)
            build.setRobotTimesAvg(robotInTimes/callInTodaySuccess);
        build.setTransfer2Human(this.tCallRecordMapper.getCallInEqualsDay(nowDateStr, 2, null));
        build.setRobotQueue(-1);//todo 逻辑如何处理
        TBucketRecord bucketRecord = this.bucketRecordMapper.selectNewest();
        if(!Objects.isNull(bucketRecord)){
            build.setRobotStreamRate(bucketRecord.getAiPercent());
            build.setHumanStreamRate(bucketRecord.getTraditionPercent());
        }
        //今日人工坐席情况
        build.setHumanInTotal(this.tCallRecordMapper.getCallInEqualsDay(nowDateStr, 0, null));
        long humanInTimes = this.tCallRecordMapper.getCallTimesEqualsDay(nowDateStr, 0);
        build.setHumanInTimes(humanInTimes);
        //获取今日已接听的人工坐席总呼入量
        int callInTodaySuccessHuman = this.tCallRecordMapper.getCallInEqualsDay(nowDateStr, 0, 1);
        if(callInTodaySuccessHuman > 0)
            build.setHumanTimesAvg(humanInTimes/callInTodaySuccessHuman);
        build.setOnlineTotal(this.tCallRecordMapper.getPersonCountByState(2));
        build.setHumanQueue(-1);//todo 逻辑如何处理
        return build;
    }


    /**
     * 校验数据
     * @param tCallRecord
     * @return
     */
    private static String checkParamsAndGetUrl(TCallRecord tCallRecord) {
        if(Objects.isNull(tCallRecord))
            throw new ServiceException("请输入正确的ID");
        Long hasParsed = tCallRecord.getHasParsed();
        if(!Objects.isNull(hasParsed) && 1== hasParsed)
            throw new ServiceException("此次通话记录已经做过转换了，无需重复转换");
        String url = tCallRecord.getUrl();
        if(StringUtils.isBlank(url))
            throw new ServiceException("此次通话记录，未记录录音文件，无法转换");
        return url;
    }
}
