package com.slibra.business.service.impl;

import java.util.Date;
import java.util.List;

import com.slibra.business.domain.TCloseTime;
import com.slibra.business.mapper.TCloseTimeMapper;
import com.slibra.common.utils.DateUtils;
import com.slibra.common.utils.SecurityUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.slibra.business.mapper.TWarningRecordMapper;
import com.slibra.business.domain.TWarningRecord;
import com.slibra.business.service.ITWarningRecordService;

/**
 * 告警记录Service业务层处理
 * 
 * @author slibra
 * @date 2025-01-16
 */
@Service
public class TWarningRecordServiceImpl implements ITWarningRecordService 
{
    @Autowired
    private TWarningRecordMapper tWarningRecordMapper;

    @Autowired
    private TCloseTimeMapper tCloseTimeMapper;

    /**
     * 查询告警记录
     * 
     * @param id 告警记录主键
     * @return 告警记录
     */
    @Override
    public TWarningRecord selectTWarningRecordById(Long id)
    {
        return tWarningRecordMapper.selectTWarningRecordById(id);
    }

    /**
     * 查询告警记录列表
     * 
     * @param tWarningRecord 告警记录
     * @return 告警记录
     */
    @Override
    public List<TWarningRecord> selectTWarningRecordList(TWarningRecord tWarningRecord)
    {
        return tWarningRecordMapper.selectTWarningRecordList(tWarningRecord);
    }

    /**
     * 新增告警记录
     * 
     * @param tWarningRecord 告警记录
     * @return 结果
     */
    @Override
    public int insertTWarningRecord(TWarningRecord tWarningRecord)
    {
        tWarningRecord.setCreateTime(DateUtils.getNowDate());
        return tWarningRecordMapper.insertTWarningRecord(tWarningRecord);
    }

    /**
     * 修改告警记录
     * 
     * @param tWarningRecord 告警记录
     * @return 结果
     */
    @Override
    public int updateTWarningRecord(TWarningRecord tWarningRecord)
    {
        tWarningRecord.setUpdateTime(DateUtils.getNowDate());
        return tWarningRecordMapper.updateTWarningRecord(tWarningRecord);
    }

    /**
     * 批量删除告警记录
     * 
     * @param ids 需要删除的告警记录主键
     * @return 结果
     */
    @Override
    public int deleteTWarningRecordByIds(Long[] ids)
    {
        return tWarningRecordMapper.deleteTWarningRecordByIds(ids);
    }

    /**
     * 删除告警记录信息
     * 
     * @param id 告警记录主键
     * @return 结果
     */
    @Override
    public int deleteTWarningRecordById(Long id)
    {
        return tWarningRecordMapper.deleteTWarningRecordById(id);
    }

    @Override
    public String addCloseRecord() {
        TCloseTime tCloseTime = new TCloseTime();
        Date nowDate = DateUtils.getNowDate();
        tCloseTime.setCreateTime(nowDate);
        tCloseTime.setCreateTime(nowDate);
        tCloseTime.setCreateBy(SecurityUtils.getUsername());
        this.tCloseTimeMapper.insertTCloseTime(tCloseTime);
        return "记录成功";
    }

    @Override
    public Integer getNewestWarningCount() {
        //获取最新的一条关闭报警记录的时间 并 查询
        return this.tWarningRecordMapper.getNewestWarningCount(this.tCloseTimeMapper.selectNewestRecord());
    }
}
